@@ -28,6 +28,8 @@
 module Array :
 sig
 
+  type 'a t = 'a array
+
   (** {6 New functions} *)
   val rev : 'a array -> 'a array
     (** Array reversal. *)
@@ -130,6 +132,14 @@ sig
   end
 #endif
 
+  val for_all2 : ('a -> 'b -> bool) -> 'a array -> 'b array -> bool
+  (** Same as {!Array.for_all}, but for a two-argument predicate.
+     @raise Invalid_argument if the two arrays have different lengths. *)
+
+  val exists2 : ('a -> 'b -> bool) -> 'a array -> 'b array -> bool
+  (** Same as {!Array.exists}, but for a two-argument predicate.
+     @raise Invalid_argument if the two arrays have different lengths. *)
+
   (** {6 Old functions} *)
 
   (** These functions are already part of the Ocaml standard library
